@@ -10,14 +10,15 @@ import errno
 import shutil
 import pickle
 import StringIO
-import rfc822
+import email.message
+import email.parser
 import tempfile
 import signal
 import time
 from hashlib import md5
 
 from zope.interface.verify import verifyClass
-from zope.interface import Interface, implements
+from zope.interface import Interface, implementer
 
 from twisted.trial import unittest
 from twisted.mail import smtp
@@ -36,7 +37,7 @@ from twisted.python import failure
 from twisted.python.filepath import FilePath
 from twisted.python import log
 from twisted.mail.relaymanager import _AttemptManager
-from twisted.test.proto_helpers import MemoryReactorClock
+from twisted.test.proto_helpers import MemoryReactorClock, StringTransport
 
 from twisted import mail
 import twisted.mail.mail
@@ -85,7 +86,6 @@ class DomainWithDefaultsTests(unittest.TestCase):
             self.assertEqual(d[x], x + 10)
             self.assertEqual(d.get(x), x + 10)
             self.assertTrue(x in d)
-            self.assertTrue(d.has_key(x))
 
         del d[2], d[4], d[6]
 
@@ -146,6 +146,25 @@ class DomainWithDefaultsTests(unittest.TestCase):
         self._stringificationTest(repr)
 
 
+    def test_has_keyDeprecation(self):
+        """
+        has_key is now deprecated.
+        """
+        sut = mail.mail.DomainWithDefaultDict({}, 'Default')
+
+        sut.has_key('anything')
+
+        message = (
+            'twisted.mail.mail.DomainWithDefaultDict.has_key was deprecated '
+            'in Twisted 16.3.0. Use the `in` keyword instead.'
+            )
+        warnings = self.flushWarnings(
+            [self.test_has_keyDeprecation])
+        self.assertEqual(1, len(warnings))
+        self.assertEqual(DeprecationWarning, warnings[0]['category'])
+        self.assertEqual(message, warnings[0]['message'])
+
+
 
 class BounceTests(unittest.TestCase):
     def setUp(self):
@@ -169,11 +188,47 @@ class BounceTests(unittest.TestCase):
 
 
 
+class BounceWithSMTPServerTests(unittest.TestCase):
+    """
+    Tests for L{twisted.mail.mail.BounceDomain} with
+    L{twisted.mail.smtp.SMTPServer}.
+    """
+    def test_rejected(self):
+        """
+        Incoming emails to a SMTP server with L{twisted.mail.mail.BounceDomain}
+        are rejected.
+        """
+        service = mail.mail.MailService()
+        domain = mail.mail.BounceDomain()
+        service.addDomain(b'foo.com', domain)
+
+        factory = mail.protocols.SMTPFactory(service)
+        protocol = factory.buildProtocol(None)
+
+        deliverer = mail.protocols.SMTPDomainDelivery(service, None, None)
+        protocol.delivery = deliverer
+
+        transport = StringTransport()
+        protocol.makeConnection(transport)
+
+        protocol.lineReceived(b'HELO baz.net')
+        protocol.lineReceived(b'MAIL FROM:<a@baz.net>')
+        protocol.lineReceived(b'RCPT TO:<any@foo.com>')
+        protocol.lineReceived(b'QUIT')
+
+        self.assertTrue(transport.disconnecting)
+        protocol.connectionLost(None)
+
+        self.assertEqual(transport.value().strip().split(b'\r\n')[-2],
+                         b'550 Cannot receive for specified address')
+
+
+
 class FileMessageTests(unittest.TestCase):
     def setUp(self):
         self.name = "fileMessage.testFile"
         self.final = "final.fileMessage.testFile"
-        self.f = file(self.name, 'w')
+        self.f = open(self.name, 'w')
         self.fp = mail.mail.FileMessage(self.f, self.name, self.final)
 
     def tearDown(self):
@@ -190,20 +245,25 @@ class FileMessageTests(unittest.TestCase):
         except:
             pass
 
+
     def testFinalName(self):
         return self.fp.eomReceived().addCallback(self._cbFinalName)
 
+
     def _cbFinalName(self, result):
         self.assertEqual(result, self.final)
         self.assertTrue(self.f.closed)
         self.assertFalse(os.path.exists(self.name))
 
+
     def testContents(self):
         contents = "first line\nsecond line\nthird line\n"
         for line in contents.splitlines():
             self.fp.lineReceived(line)
         self.fp.eomReceived()
-        self.assertEqual(file(self.final).read(), contents)
+        with open(self.final) as f:
+            self.assertEqual(f.read(), contents)
+
 
     def testInterrupted(self):
         contents = "first line\nsecond line\n"
@@ -213,10 +273,13 @@ class FileMessageTests(unittest.TestCase):
         self.assertFalse(os.path.exists(self.name))
         self.assertFalse(os.path.exists(self.final))
 
+
+
 class MailServiceTests(unittest.TestCase):
     def setUp(self):
         self.service = mail.mail.MailService()
 
+
     def testFactories(self):
         f = self.service.getPOP3Factory()
         self.assertTrue(isinstance(f, protocol.ServerFactory))
@@ -230,6 +293,7 @@ class MailServiceTests(unittest.TestCase):
         self.assertTrue(isinstance(f, protocol.ServerFactory))
         self.assertTrue(f.buildProtocol(('127.0.0.1', 12345)), smtp.ESMTP)
 
+
     def testPortals(self):
         o1 = object()
         o2 = object()
@@ -240,6 +304,7 @@ class MailServiceTests(unittest.TestCase):
         self.assertTrue(self.service.defaultPortal() is o2)
 
 
+
 class StringListMailboxTests(unittest.TestCase):
     """
     Tests for L{StringListMailbox}, an in-memory only implementation of
@@ -325,16 +390,22 @@ class FailingMaildirMailboxAppendMessageTask(mail.maildir._MaildirMailboxAppendM
     _openstate = True
     _writestate = True
     _renamestate = True
+
+
     def osopen(self, fn, attr, mode):
         if self._openstate:
             return os.open(fn, attr, mode)
         else:
             raise OSError(errno.EPERM, "Faked Permission Problem")
+
+
     def oswrite(self, fh, data):
         if self._writestate:
             return os.write(fh, data)
         else:
             raise OSError(errno.ENOSPC, "Faked Space problem")
+
+
     def osrename(self, oldname, newname):
         if self._renamestate:
             return os.rename(oldname, newname)
@@ -342,6 +413,7 @@ class FailingMaildirMailboxAppendMessageTask(mail.maildir._MaildirMailboxAppendM
             raise OSError(errno.EPERM, "Faked Permission Problem")
 
 
+
 class _AppendTestMixin(object):
     """
     Mixin for L{MaildirMailbox.appendMessage} test cases which defines a helper
@@ -384,7 +456,7 @@ class MaildirAppendStringTests(unittest.TestCase, _AppendTestMixin):
         Change the behavior of future C{rename}, C{write}, or C{open} calls made
         by the mailbox C{mbox}.
 
-        @param rename: If not C{None}, a new value for the C{_renamestate}
+        @param rename: If not L{None}, a new value for the C{_renamestate}
             attribute of the mailbox's append factory.  The original value will
             be restored at the end of the test.
 
@@ -491,9 +563,11 @@ class MaildirTests(unittest.TestCase):
         self.d = self.mktemp()
         mail.maildir.initializeMaildir(self.d)
 
+
     def tearDown(self):
         shutil.rmtree(self.d)
 
+
     def testInitializer(self):
         d = self.d
         trash = os.path.join(d, '.Trash')
@@ -544,9 +618,8 @@ class MaildirTests(unittest.TestCase):
         # Toss a few files into the mailbox
         i = 1
         for f in msgs:
-            fObj = file(j(self.d, f), 'w')
-            fObj.write('x' * i)
-            fObj.close()
+            with open(j(self.d, f), 'w') as fObj:
+                fObj.write('x' * i)
             i = i + 1
 
         mb = mail.maildir.MaildirMailbox(self.d)
@@ -706,12 +779,11 @@ class MaildirDirdbmDomainTests(unittest.TestCase):
 
 
 
+@implementer(mail.mail.IAliasableDomain)
 class StubAliasableDomain(object):
     """
     Minimal testable implementation of IAliasableDomain.
     """
-    implements(mail.mail.IAliasableDomain)
-
     def exists(self, user):
         """
         No test coverage for invocations of this method on domain objects,
@@ -745,6 +817,7 @@ class StubAliasableDomain(object):
         self.aliasGroup = aliases
 
 
+
 class ServiceDomainTests(unittest.TestCase):
     def setUp(self):
         self.S = mail.mail.MailService()
@@ -758,6 +831,7 @@ class ServiceDomainTests(unittest.TestCase):
         domain.addUser('user', 'password')
         self.S.addDomain('test.domain', domain)
 
+
     def tearDown(self):
         shutil.rmtree(self.tmpdir)
 
@@ -775,15 +849,17 @@ class ServiceDomainTests(unittest.TestCase):
 
 
     def testReceivedHeader(self):
-         hdr = self.D.receivedHeader(
-             ('remotehost', '123.232.101.234'),
-             smtp.Address('<someguy@someplace>'),
-             ['user@host.name']
-         )
-         fp = StringIO.StringIO(hdr)
-         m = rfc822.Message(fp)
-         self.assertEqual(len(m.items()), 1)
-         self.assertIn('Received', m)
+        hdr = self.D.receivedHeader(
+            ('remotehost', '123.232.101.234'),
+            smtp.Address('<someguy@someplace>'),
+            ['user@host.name']
+        )
+        fp = StringIO.StringIO(hdr)
+        emailParser = email.parser.Parser()
+        m = emailParser.parse(fp)
+        self.assertEqual(len(m.items()), 1)
+        self.assertIn('Received', m)
+
 
     def testValidateTo(self):
         user = smtp.User('user@test.domain', 'helo', None, 'wherever@whatever')
@@ -791,21 +867,25 @@ class ServiceDomainTests(unittest.TestCase):
             ).addCallback(self._cbValidateTo
             )
 
+
     def _cbValidateTo(self, result):
         self.assertTrue(callable(result))
 
+
     def testValidateToBadUsername(self):
         user = smtp.User('resu@test.domain', 'helo', None, 'wherever@whatever')
         return self.assertFailure(
             defer.maybeDeferred(self.D.validateTo, user),
             smtp.SMTPBadRcpt)
 
+
     def testValidateToBadDomain(self):
         user = smtp.User('user@domain.test', 'helo', None, 'wherever@whatever')
         return self.assertFailure(
             defer.maybeDeferred(self.D.validateTo, user),
             smtp.SMTPBadRcpt)
 
+
     def testValidateFrom(self):
         helo = ('hostname', '127.0.0.1')
         origin = smtp.Address('<user@hostname>')
@@ -824,6 +904,8 @@ class ServiceDomainTests(unittest.TestCase):
             self.D.validateFrom, None, origin
         )
 
+
+
 class VirtualPOP3Tests(unittest.TestCase):
     def setUp(self):
         self.tmpdir = self.mktemp()
@@ -840,49 +922,58 @@ class VirtualPOP3Tests(unittest.TestCase):
         self.P.service = self.S
         self.P.magic = '<unit test magic>'
 
+
     def tearDown(self):
         shutil.rmtree(self.tmpdir)
 
+
     def testAuthenticateAPOP(self):
         resp = md5(self.P.magic + 'password').hexdigest()
         return self.P.authenticateUserAPOP('user', resp
             ).addCallback(self._cbAuthenticateAPOP
             )
 
+
     def _cbAuthenticateAPOP(self, result):
         self.assertEqual(len(result), 3)
         self.assertEqual(result[0], pop3.IMailbox)
         self.assertTrue(pop3.IMailbox.providedBy(result[1]))
         result[2]()
 
+
     def testAuthenticateIncorrectUserAPOP(self):
         resp = md5(self.P.magic + 'password').hexdigest()
         return self.assertFailure(
             self.P.authenticateUserAPOP('resu', resp),
             cred.error.UnauthorizedLogin)
 
+
     def testAuthenticateIncorrectResponseAPOP(self):
         resp = md5('wrong digest').hexdigest()
         return self.assertFailure(
             self.P.authenticateUserAPOP('user', resp),
             cred.error.UnauthorizedLogin)
 
+
     def testAuthenticatePASS(self):
         return self.P.authenticateUserPASS('user', 'password'
             ).addCallback(self._cbAuthenticatePASS
             )
 
+
     def _cbAuthenticatePASS(self, result):
         self.assertEqual(len(result), 3)
         self.assertEqual(result[0], pop3.IMailbox)
         self.assertTrue(pop3.IMailbox.providedBy(result[1]))
         result[2]()
 
+
     def testAuthenticateBadUserPASS(self):
         return self.assertFailure(
             self.P.authenticateUserPASS('resu', 'password'),
             cred.error.UnauthorizedLogin)
 
+
     def testAuthenticateBadPasswordPASS(self):
         return self.assertFailure(
             self.P.authenticateUserPASS('user', 'wrong password'),
@@ -940,11 +1031,10 @@ class RelayerTests(unittest.TestCase):
         self.messageFiles = []
         for i in range(10):
             name = os.path.join(self.tmpdir, 'body-%d' % (i,))
-            f = file(name + '-H', 'w')
-            pickle.dump(['from-%d' % (i,), 'to-%d' % (i,)], f)
-            f.close()
+            with open(name + '-H', 'w') as f:
+                pickle.dump(['from-%d' % (i,), 'to-%d' % (i,)], f)
 
-            f = file(name + '-D', 'w')
+            f = open(name + '-D', 'w')
             f.write(name)
             f.seek(0, 0)
             self.messageFiles.append(name)
@@ -952,21 +1042,25 @@ class RelayerTests(unittest.TestCase):
         self.R = mail.relay.RelayerMixin()
         self.R.loadMessages(self.messageFiles)
 
+
     def tearDown(self):
         shutil.rmtree(self.tmpdir)
 
+
     def testMailFrom(self):
         for i in range(10):
             self.assertEqual(self.R.getMailFrom(), 'from-%d' % (i,))
             self.R.sentMail(250, None, None, None, None)
         self.assertEqual(self.R.getMailFrom(), None)
 
+
     def testMailTo(self):
         for i in range(10):
             self.assertEqual(self.R.getMailTo(), ['to-%d' % (i,)])
             self.R.sentMail(250, None, None, None, None)
         self.assertEqual(self.R.getMailTo(), None)
 
+
     def testMailData(self):
         for i in range(10):
             name = os.path.join(self.tmpdir, 'body-%d' % (i,))
@@ -974,21 +1068,28 @@ class RelayerTests(unittest.TestCase):
             self.R.sentMail(250, None, None, None, None)
         self.assertEqual(self.R.getMailData(), None)
 
+
+
 class Manager:
     def __init__(self):
         self.success = []
         self.failure = []
         self.done = []
 
+
     def notifySuccess(self, factory, message):
         self.success.append((factory, message))
 
+
     def notifyFailure(self, factory, message):
         self.failure.append((factory, message))
 
+
     def notifyDone(self, factory):
         self.done.append(factory)
 
+
+
 class ManagedRelayerTests(unittest.TestCase):
     def setUp(self):
         self.manager = Manager()
@@ -999,6 +1100,7 @@ class ManagedRelayerTests(unittest.TestCase):
         self.relay.names = self.messages[:]
         self.relay.factory = self.factory
 
+
     def testSuccessfulSentMail(self):
         for i in self.messages:
             self.relay.sentMail(250, None, None, None, None)
@@ -1008,6 +1110,7 @@ class ManagedRelayerTests(unittest.TestCase):
             [(self.factory, m) for m in self.messages]
         )
 
+
     def testFailedSentMail(self):
         for i in self.messages:
             self.relay.sentMail(550, None, None, None, None)
@@ -1017,10 +1120,13 @@ class ManagedRelayerTests(unittest.TestCase):
             [(self.factory, m) for m in self.messages]
         )
 
+
     def testConnectionLost(self):
         self.relay.connectionLost(failure.Failure(Exception()))
         self.assertEqual(self.manager.done, [self.factory])
 
+
+
 class DirectoryQueueTests(unittest.TestCase):
     def setUp(self):
         # This is almost a test case itself.
@@ -1030,15 +1136,17 @@ class DirectoryQueueTests(unittest.TestCase):
         self.queue.noisy = False
         for m in range(25):
             hdrF, msgF = self.queue.createNewMessage()
-            pickle.dump(['header', m], hdrF)
-            hdrF.close()
+            with hdrF:
+                pickle.dump(['header', m], hdrF)
             msgF.lineReceived('body: %d' % (m,))
             msgF.eomReceived()
         self.queue.readDirectory()
 
+
     def tearDown(self):
         shutil.rmtree(self.tmpdir)
 
+
     def testWaiting(self):
         self.assertTrue(self.queue.hasWaiting())
         self.assertEqual(len(self.queue.getWaiting()), 25)
@@ -1050,6 +1158,7 @@ class DirectoryQueueTests(unittest.TestCase):
         self.queue.setWaiting(waiting[0])
         self.assertEqual(len(self.queue.getWaiting()), 25)
 
+
     def testRelaying(self):
         for m in self.queue.getWaiting():
             self.queue.setRelaying(m)
@@ -1065,6 +1174,7 @@ class DirectoryQueueTests(unittest.TestCase):
         self.assertEqual(len(self.queue.getWaiting()), 1)
         self.assertEqual(len(self.queue.getRelayed()), 24)
 
+
     def testDone(self):
         msg = self.queue.getWaiting()[0]
         self.queue.setRelaying(msg)
@@ -1076,6 +1186,7 @@ class DirectoryQueueTests(unittest.TestCase):
         self.assertFalse(msg in self.queue.getWaiting())
         self.assertFalse(msg in self.queue.getRelayed())
 
+
     def testEnvelope(self):
         envelopes = []
 
@@ -1098,6 +1209,7 @@ class TestAuthority(common.ResolverBase):
         common.ResolverBase.__init__(self)
         self.addresses = {}
 
+
     def _lookup(self, name, cls, type, timeout = None):
         if name in self.addresses and type == dns.MX:
             results = []
@@ -1109,6 +1221,8 @@ class TestAuthority(common.ResolverBase):
             return defer.succeed((results, [], []))
         return defer.fail(failure.Failure(dns.DomainError(name)))
 
+
+
 def setUpDNS(self):
     self.auth = TestAuthority()
     factory = server.DNSServerFactory([self.auth])
@@ -1126,6 +1240,7 @@ def setUpDNS(self):
     self.resolver = client.Resolver(servers=[('127.0.0.1', portNumber)])
 
 
+
 def tearDownDNS(self):
     dl = []
     dl.append(defer.maybeDeferred(self.port.stopListening))
@@ -1136,6 +1251,8 @@ def tearDownDNS(self):
         pass
     return defer.DeferredList(dl)
 
+
+
 class MXTests(unittest.TestCase):
     """
     Tests for L{mail.relaymanager.MXCalculator}.
@@ -1145,6 +1262,7 @@ class MXTests(unittest.TestCase):
         self.clock = task.Clock()
         self.mx = mail.relaymanager.MXCalculator(self.resolver, self.clock)
 
+
     def tearDown(self):
         return tearDownDNS(self)
 
@@ -1162,14 +1280,17 @@ class MXTests(unittest.TestCase):
         self.auth.addresses['test.domain'] = ['the.email.test.domain']
         return self.mx.getMX('test.domain').addCallback(self._cbSimpleSuccess)
 
+
     def _cbSimpleSuccess(self, mx):
         self.assertEqual(mx.preference, 0)
         self.assertEqual(str(mx.name), 'the.email.test.domain')
 
+
     def testSimpleFailure(self):
         self.mx.fallbackToDomain = False
         return self.assertFailure(self.mx.getMX('test.domain'), IOError)
 
+
     def testSimpleFailureWithFallback(self):
         return self.assertFailure(self.mx.getMX('test.domain'), DNSLookupError)
 
@@ -1507,6 +1628,7 @@ class MXTests(unittest.TestCase):
             ).addCallback(self._cbManyRecordsSuccessfulLookup
             )
 
+
     def _cbManyRecordsSuccessfulLookup(self, mx):
         self.assertTrue(str(mx.name).split('.', 1)[0] in ('mx1', 'mx2', 'mx3'))
         self.mx.markBad(str(mx.name))
@@ -1514,6 +1636,7 @@ class MXTests(unittest.TestCase):
             ).addCallback(self._cbManyRecordsDifferentResult, mx
             )
 
+
     def _cbManyRecordsDifferentResult(self, nextMX, mx):
         self.assertNotEqual(str(mx.name), str(nextMX.name))
         self.mx.markBad(str(nextMX.name))
@@ -1522,6 +1645,7 @@ class MXTests(unittest.TestCase):
             ).addCallback(self._cbManyRecordsLastResult, mx, nextMX
             )
 
+
     def _cbManyRecordsLastResult(self, lastMX, mx, nextMX):
         self.assertNotEqual(str(mx.name), str(lastMX.name))
         self.assertNotEqual(str(nextMX.name), str(lastMX.name))
@@ -1533,13 +1657,17 @@ class MXTests(unittest.TestCase):
             ).addCallback(self._cbManyRecordsRepeatSpecificResult, nextMX
             )
 
+
     def _cbManyRecordsRepeatSpecificResult(self, againMX, nextMX):
         self.assertEqual(str(againMX.name), str(nextMX.name))
 
+
+
 class LiveFireExerciseTests(unittest.TestCase):
     if interfaces.IReactorUDP(reactor, None) is None:
         skip = "UDP support is required to determining MX records"
 
+
     def setUp(self):
         setUpDNS(self)
         self.tmpdirs = [
@@ -1547,12 +1675,14 @@ class LiveFireExerciseTests(unittest.TestCase):
             'destinationDomain', 'destinationQueue'
         ]
 
+
     def tearDown(self):
         for d in self.tmpdirs:
             if os.path.exists(d):
                 shutil.rmtree(d)
         return tearDownDNS(self)
 
+
     def testLocalDelivery(self):
         service = mail.mail.MailService()
         service.smtpPortal.registerChecker(cred.checkers.AllowAnonymousAccess())
@@ -1587,7 +1717,7 @@ class LiveFireExerciseTests(unittest.TestCase):
         def finished(ign):
             mbox = domain.requestAvatar('user', None, pop3.IMailbox)[1]
             msg = mbox.getMessage(0).read()
-            self.failIfEqual(msg.find('This is the message'), -1)
+            self.assertNotEqual(msg.find('This is the message'), -1)
 
             return self.smtpServer.stopListening()
         done.addCallback(finished)
@@ -1656,7 +1786,7 @@ class LiveFireExerciseTests(unittest.TestCase):
             def delivered(ign):
                 mbox = domain.requestAvatar('user', None, pop3.IMailbox)[1]
                 msg = mbox.getMessage(0).read()
-                self.failIfEqual(msg.find('This is the message'), -1)
+                self.assertNotEqual(msg.find('This is the message'), -1)
 
                 self.insServer.stopListening()
                 self.destServer.stopListening()
@@ -1683,16 +1813,21 @@ class LineBufferMessage:
         self.eom = False
         self.lost = False
 
+
     def lineReceived(self, line):
         self.lines.append(line)
 
+
     def eomReceived(self):
         self.eom = True
         return defer.succeed('<Whatever>')
 
+
     def connectionLost(self):
         self.lost = True
 
+
+
 class AliasTests(unittest.TestCase):
     lines = [
         'First line',
@@ -1705,6 +1840,7 @@ class AliasTests(unittest.TestCase):
     def setUp(self):
         aliasFile.seek(0)
 
+
     def testHandle(self):
         result = {}
         lines = [
@@ -1723,6 +1859,7 @@ class AliasTests(unittest.TestCase):
         self.assertEqual(result['moreusers'], [':/etc/include/filename'])
         self.assertEqual(result['multiuser'], ['first@host', 'second@host', 'last@anotherhost'])
 
+
     def testFileLoader(self):
         domains = {'': object()}
         result = mail.alias.loadAliasFile(domains, fp=aliasFile)
@@ -1732,22 +1869,23 @@ class AliasTests(unittest.TestCase):
         group = result['testuser']
         s = str(group)
         for a in ('address1', 'address2', 'address3', 'continuation@address', '/bin/process/this'):
-            self.failIfEqual(s.find(a), -1)
+            self.assertNotEqual(s.find(a), -1)
         self.assertEqual(len(group), 5)
 
         group = result['usertwo']
         s = str(group)
         for a in ('thisaddress', 'thataddress', 'lastaddress'):
-            self.failIfEqual(s.find(a), -1)
+            self.assertNotEqual(s.find(a), -1)
         self.assertEqual(len(group), 3)
 
         group = result['lastuser']
         s = str(group)
         self.assertEqual(s.find('/includable'), -1)
         for a in ('/filename', 'program', 'address'):
-            self.failIfEqual(s.find(a), -1, '%s not found' % a)
+            self.assertNotEqual(s.find(a), -1, '%s not found' % a)
         self.assertEqual(len(group), 3)
 
+
     def testMultiWrapper(self):
         msgs = LineBufferMessage(), LineBufferMessage(), LineBufferMessage()
         msg = mail.alias.MultiWrapper(msgs)
@@ -1756,12 +1894,14 @@ class AliasTests(unittest.TestCase):
             msg.lineReceived(L)
         return msg.eomReceived().addCallback(self._cbMultiWrapper, msgs)
 
+
     def _cbMultiWrapper(self, ignored, msgs):
         for m in msgs:
             self.assertTrue(m.eom)
             self.assertFalse(m.lost)
             self.assertEqual(self.lines, m.lines)
 
+
     def testFileAlias(self):
         tmpfile = self.mktemp()
         a = mail.alias.FileAlias(tmpfile, None, None)
@@ -1771,8 +1911,10 @@ class AliasTests(unittest.TestCase):
             m.lineReceived(l)
         return m.eomReceived().addCallback(self._cbTestFileAlias, tmpfile)
 
+
     def _cbTestFileAlias(self, ignored, tmpfile):
-        lines = file(tmpfile).readlines()
+        with open(tmpfile) as f:
+            lines = f.readlines()
         self.assertEqual([L[:-1] for L in lines], self.lines)
 
 
@@ -1804,7 +1946,6 @@ class AddressAliasTests(unittest.TestCase):
     """
     Tests for L{twisted.mail.alias.AddressAlias}.
     """
-
     def setUp(self):
         """
         Setup an L{AddressAlias}.
@@ -1833,14 +1974,14 @@ class AddressAliasTests(unittest.TestCase):
     def test_resolve(self):
         """
         L{resolve} will look for additional aliases when an C{aliasmap}
-        dictionary is passed, and returns C{None} if none were found.
+        dictionary is passed, and returns L{None} if none were found.
         """
         self.assertEqual(self.alias.resolve({self.address: 'bar'}), None)
 
 
     def test_resolveWithoutAliasmap(self):
         """
-        L{resolve} returns C{None} when the alias could not be found in the
+        L{resolve} returns L{None} when the alias could not be found in the
         C{aliasmap} and no L{mail.smtp.User} with this alias exists either.
         """
         self.assertEqual(self.alias.resolve({}), None)
@@ -1854,7 +1995,7 @@ class DummyProcess(object):
 
 class MockProcessAlias(mail.alias.ProcessAlias):
     """
-    A alias processor that doesn't actually launch processes.
+    An alias processor that doesn't actually launch processes.
     """
 
     def spawnProcess(self, proto, program, path):
@@ -1986,7 +2127,7 @@ rm -f process.alias.out
 while read i; do
     echo $i >> process.alias.out
 done""")
-        os.chmod(sh.path, 0700)
+        os.chmod(sh.path, 0o700)
         a = mail.alias.ProcessAlias(sh.path, None, None)
         m = a.createMessageReceiver()
 
@@ -1994,7 +2135,8 @@ done""")
             m.lineReceived(l)
 
         def _cbProcessAlias(ignored):
-            lines = file('process.alias.out').readlines()
+            with open('process.alias.out') as f:
+                lines = f.readlines()
             self.assertEqual([L[:-1] for L in lines], self.lines)
 
         return m.eomReceived().addCallback(_cbProcessAlias)
@@ -2160,14 +2302,12 @@ done""")
 
 
 
-
-
-
 class TestDomain:
     def __init__(self, aliases, users):
         self.aliases = aliases
         self.users = users
 
+
     def exists(self, user, memo=None):
         user = user.dest.local
         if user in self.users:
@@ -2184,28 +2324,6 @@ class TestDomain:
 
 
 
-class SSLContextFactoryTests(unittest.TestCase):
-    """
-    Tests for twisted.mail.protocols.SSLContextFactory.
-    """
-    def test_deprecation(self):
-        """
-        Accessing L{twisted.mail.protocols.SSLContextFactory} emits a
-        deprecation warning recommending the use of the more general SSL context
-        factory from L{twisted.internet.ssl}.
-        """
-        mail.protocols.SSLContextFactory
-        warningsShown = self.flushWarnings([self.test_deprecation])
-        self.assertEqual(len(warningsShown), 1)
-        self.assertIdentical(warningsShown[0]['category'], DeprecationWarning)
-        self.assertEqual(
-            warningsShown[0]['message'],
-            'twisted.mail.protocols.SSLContextFactory was deprecated in '
-            'Twisted 12.2.0: Use twisted.internet.ssl.'
-            'DefaultOpenSSLContextFactory instead.')
-
-
-
 class DummyQueue(object):
     """
     A fake relay queue to use for testing.
@@ -2324,7 +2442,7 @@ class _AttemptManagerTests(unittest.TestCase):
     @type reactor: L{MemoryReactorClock}
     @ivar reactor: The reactor used for test purposes.
 
-    @type eventLog: L{types.NoneType} or L{dict} of L{bytes} -> L{object}
+    @type eventLog: L{None} or L{dict} of L{bytes} -> L{object}
     @ivar eventLog: Information about the last informational log message
         generated or none if no log message has been generated.
 
@@ -2372,24 +2490,20 @@ class _AttemptManagerTests(unittest.TestCase):
         self.noisyMessage = os.path.join(self.tmpdir, noisyBaseName)
         self.quietMessage = os.path.join(self.tmpdir, quietBaseName)
 
-        message = file(self.noisyMessage+'-D', "w")
-        message.close()
+        open(self.noisyMessage+'-D', "w").close()
 
-        message = file(self.quietMessage+'-D', "w")
-        message.close()
+        open(self.quietMessage+'-D', "w").close()
 
         self.noisyAttemptMgr.manager.managed['noisyRelayer'] = [
                 noisyBaseName]
         self.quietAttemptMgr.manager.managed['quietRelayer'] = [
                 quietBaseName]
 
-        envelope = file(self.noisyMessage+'-H', 'w')
-        pickle.dump(['from-noisy@domain', 'to-noisy@domain'], envelope)
-        envelope.close()
+        with open(self.noisyMessage+'-H', 'w') as envelope:
+            pickle.dump(['from-noisy@domain', 'to-noisy@domain'], envelope)
 
-        envelope = file(self.quietMessage+'-H', 'w')
-        pickle.dump(['from-quiet@domain', 'to-quiet@domain'], envelope)
-        envelope.close()
+        with open(self.quietMessage+'-H', 'w') as envelope:
+            pickle.dump(['from-quiet@domain', 'to-quiet@domain'], envelope)
 
 
     def tearDown(self):
